@@ -1,33 +1,39 @@
-use std::cell::RefCell;
-use std::fmt::{self, Debug, Formatter};
+use std::cell::{Cell, RefCell};
+use std::fmt::Debug;
 use std::io;
 use std::os::unix::io::AsRawFd;
 use std::rc::Rc;
 use std::sync::atomic::{AtomicBool, Ordering};
 use std::sync::Arc;
-use std::time::Duration;
+use std::time::{Duration, Instant};
+
+use slotmap::SlotMap;
 
-use crate::list::SourceList;
 use crate::sources::{Dispatcher, EventSource, Idle, IdleDispatcher};
-use crate::{Poll, PostAction, TokenFactory};
+use crate::{EventDispatcher, InsertError, Poll, PostAction, TokenFactory};
 
 type IdleCallback<'i, Data> = Rc<RefCell<dyn IdleDispatcher<Data> + 'i>>;
 
+slotmap::new_key_type! {
+    pub(crate) struct CalloopKey;
+}
+
 /// A token representing a registration in the [`EventLoop`].
 ///
 /// This token is given to you by the [`EventLoop`] when an [`EventSource`] is inserted or
 /// a [`Dispatcher`] is registered. You can use it to [disable](LoopHandle#method.disable),
 /// [enable](LoopHandle#method.enable), [update`](LoopHandle#method.update),
 /// [remove](LoopHandle#method.remove) or [kill](LoopHandle#method.kill) it.
-#[derive(Debug, PartialEq)]
+#[derive(Debug, PartialEq, Clone, Copy)]
 pub struct RegistrationToken {
-    id: u32,
+    key: CalloopKey,
 }
 
 pub(crate) struct LoopInner<'l, Data> {
     pub(crate) poll: RefCell<Poll>,
-    pub(crate) sources: RefCell<SourceList<'l, Data>>,
+    pub(crate) sources: RefCell<SlotMap<CalloopKey, Rc<dyn EventDispatcher<Data> + 'l>>>,
     idles: RefCell<Vec<IdleCallback<'l, Data>>>,
+    pending_action: Cell<PostAction>,
 }
 
 /// An handle to an event loop
@@ -40,13 +46,14 @@ pub struct LoopHandle<'l, Data> {
 }
 
 impl<'l, Data> std::fmt::Debug for LoopHandle<'l, Data> {
+    #[cfg_attr(coverage, no_coverage)]
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         f.write_str("LoopHandle { ... }")
     }
 }
 
-#[cfg(not(tarpaulin_include))]
 impl<'l, Data> Clone for LoopHandle<'l, Data> {
+    #[cfg_attr(coverage, no_coverage)]
     fn clone(&self) -> Self {
         LoopHandle {
             inner: self.inner.clone(),
@@ -54,42 +61,6 @@ impl<'l, Data> Clone for LoopHandle<'l, Data> {
     }
 }
 
-/// An error generated when trying to insert an event source
-pub struct InsertError<E> {
-    /// The source that could not be inserted
-    pub source: E,
-    /// The generated error
-    pub error: io::Error,
-}
-
-#[cfg(not(tarpaulin_include))]
-impl<E> Debug for InsertError<E> {
-    fn fmt(&self, formatter: &mut Formatter) -> Result<(), fmt::Error> {
-        write!(formatter, "{:?}", self.error)
-    }
-}
-
-#[cfg(not(tarpaulin_include))]
-impl<E> std::fmt::Display for InsertError<E> {
-    fn fmt(&self, formatter: &mut Formatter) -> Result<(), fmt::Error> {
-        write!(formatter, "{}", self.error)
-    }
-}
-
-#[cfg(not(tarpaulin_include))]
-impl<E> From<InsertError<E>> for io::Error {
-    fn from(e: InsertError<E>) -> io::Error {
-        e.error
-    }
-}
-
-#[cfg(not(tarpaulin_include))]
-impl<E> std::error::Error for InsertError<E> {
-    fn source(&self) -> Option<&(dyn std::error::Error + 'static)> {
-        self.error.source()
-    }
-}
-
 impl<'l, Data> LoopHandle<'l, Data> {
     /// Inserts a new event source in the loop.
     ///
@@ -111,7 +82,7 @@ impl<'l, Data> LoopHandle<'l, Data> {
         self.register_dispatcher(dispatcher.clone())
             .map_err(|error| InsertError {
                 error,
-                source: dispatcher.into_source_inner(),
+                inserted: dispatcher.into_source_inner(),
             })
     }
 
@@ -123,25 +94,25 @@ impl<'l, Data> LoopHandle<'l, Data> {
     pub fn register_dispatcher<S>(
         &self,
         dispatcher: Dispatcher<'l, S, Data>,
-    ) -> io::Result<RegistrationToken>
+    ) -> crate::Result<RegistrationToken>
     where
         S: EventSource + 'l,
     {
         let mut sources = self.inner.sources.borrow_mut();
         let mut poll = self.inner.poll.borrow_mut();
 
-        let id = sources.add_source(dispatcher.clone_as_event_dispatcher());
+        let key = sources.insert(dispatcher.clone_as_event_dispatcher());
         let ret = sources
-            .get_dispatcher(id)
+            .get(key)
             .unwrap()
-            .register(&mut *poll, &mut TokenFactory::new(id));
+            .register(&mut *poll, &mut TokenFactory::new(key));
 
         if let Err(error) = ret {
-            sources.del_source(id).expect("Source was just inserted?!");
+            sources.remove(key).expect("Source was just inserted?!");
             return Err(error);
         }
 
-        Ok(RegistrationToken { id })
+        Ok(RegistrationToken { key })
     }
 
     /// Inserts an idle callback.
@@ -163,80 +134,57 @@ impl<'l, Data> LoopHandle<'l, Data> {
     ///
     /// This previously disabled source will start generating events again.
     ///
-    /// **Note**: This cannot be done from within the callback of the same source.
-    pub fn enable(&self, token: &RegistrationToken) -> io::Result<()> {
-        self.inner
-            .sources
-            .borrow()
-            .get_dispatcher(token.id)
-            .expect("Attempting to access a non-existent source?!")
-            .register(
+    /// **Note:** this cannot be done from within the source callback.
+    pub fn enable(&self, token: &RegistrationToken) -> crate::Result<()> {
+        if let Some(source) = self.inner.sources.borrow().get(token.key) {
+            source.register(
                 &mut *self.inner.poll.borrow_mut(),
-                &mut TokenFactory::new(token.id),
-            )
+                &mut TokenFactory::new(token.key),
+            )?;
+        }
+        Ok(())
     }
 
     /// Makes this source update its registration.
     ///
     /// If after accessing the source you changed its parameters in a way that requires
     /// updating its registration.
-    ///
-    /// **Note**: This cannot be done from within the callback of the same source.
-    pub fn update(&self, token: &RegistrationToken) -> io::Result<()> {
-        self.inner
-            .sources
-            .borrow()
-            .get_dispatcher(token.id)
-            .expect("Attempting to access a non-existent source?!")
-            .reregister(
+    pub fn update(&self, token: &RegistrationToken) -> crate::Result<()> {
+        if let Some(source) = self.inner.sources.borrow().get(token.key) {
+            if !source.reregister(
                 &mut *self.inner.poll.borrow_mut(),
-                &mut TokenFactory::new(token.id),
-            )
+                &mut TokenFactory::new(token.key),
+            )? {
+                // we are in a callback, store for later processing
+                self.inner.pending_action.set(PostAction::Reregister);
+            }
+        }
+        Ok(())
     }
 
     /// Disables this event source.
     ///
     /// The source remains in the event loop, but it'll no longer generate events
-    ///
-    /// **Note**: This cannot be done from within the callback of the same source.
-    pub fn disable(&self, token: &RegistrationToken) -> io::Result<()> {
-        self.inner
-            .sources
-            .borrow()
-            .get_dispatcher(token.id)
-            .expect("Attempting to access a non-existent source?!")
-            .unregister(&mut *self.inner.poll.borrow_mut())
-    }
-
-    /// Removes this source from the event loop.
-    ///
-    /// **Note**: This cannot be done from within the callback of the same source.
-    pub fn remove(&self, token: RegistrationToken) {
-        let source = self
-            .inner
-            .sources
-            .borrow_mut()
-            .del_source(token.id)
-            .expect("Attempting to remove a non-existent source?!");
-
-        if let Err(e) = source.unregister(&mut self.inner.poll.borrow_mut()) {
-            log::warn!(
-                "[calloop] Failed to unregister source from the polling system: {:?}",
-                e
-            );
+    pub fn disable(&self, token: &RegistrationToken) -> crate::Result<()> {
+        if let Some(source) = self.inner.sources.borrow().get(token.key) {
+            if !source.unregister(&mut *self.inner.poll.borrow_mut())? {
+                // we are in a callback, store for later processing
+                self.inner.pending_action.set(PostAction::Disable);
+            }
         }
+        Ok(())
     }
 
     /// Removes this source from the event loop.
-    ///
-    /// **Note**: This can be done from within the callback of the same source.
-    pub fn kill(&self, token: RegistrationToken) {
-        let _source = self
-            .inner
-            .sources
-            .borrow_mut()
-            .del_source(token.id)
-            .expect("Attempting to remove a non-existent source?!");
+    pub fn remove(&self, token: RegistrationToken) {
+        if let Some(source) = self.inner.sources.borrow_mut().remove(token.key) {
+            if let Err(e) = source.unregister(&mut self.inner.poll.borrow_mut()) {
+                log::warn!(
+                    "[calloop] Failed to unregister source from the polling system: {:?}",
+                    e
+                );
+            }
+        }
     }
 
     /// Wrap an IO object into an async adapter
@@ -246,7 +194,7 @@ impl<'l, Data> LoopHandle<'l, Data> {
     ///
     /// The produced futures can be polled in any executor, and notably the one provided by
     /// calloop.
-    pub fn adapt_io<F: AsRawFd>(&self, fd: F) -> std::io::Result<crate::io::Async<'l, F>> {
+    pub fn adapt_io<F: AsRawFd>(&self, fd: F) -> crate::Result<crate::io::Async<'l, F>> {
         crate::io::Async::new(self.inner.clone(), fd)
     }
 }
@@ -261,6 +209,7 @@ pub struct EventLoop<'l, Data> {
 }
 
 impl<'l, Data> std::fmt::Debug for EventLoop<'l, Data> {
+    #[cfg_attr(coverage, no_coverage)]
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         f.write_str("EventLoop { ... }")
     }
@@ -270,13 +219,29 @@ impl<'l, Data> EventLoop<'l, Data> {
     /// Create a new event loop
     ///
     /// Fails if the initialization of the polling system failed.
-    pub fn try_new() -> io::Result<Self> {
-        let poll = Poll::new()?;
+    pub fn try_new() -> crate::Result<Self> {
+        Self::inner_new(false)
+    }
+
+    /// Create a new event loop in high precision mode
+    ///
+    /// On some platforms it requires to setup more resources to enable high-precision
+    /// (sub millisecond) capabilities, so you should use this constructor if you need
+    /// this kind of precision.
+    ///
+    /// Fails if the initialization of the polling system failed.
+    pub fn try_new_high_precision() -> crate::Result<Self> {
+        Self::inner_new(true)
+    }
+
+    fn inner_new(high_precision: bool) -> crate::Result<Self> {
+        let poll = Poll::new(high_precision)?;
         let handle = LoopHandle {
             inner: Rc::new(LoopInner {
                 poll: RefCell::new(poll),
-                sources: RefCell::new(SourceList::new()),
+                sources: RefCell::new(SlotMap::with_key()),
                 idles: RefCell::new(Vec::new()),
+                pending_action: Cell::new(PostAction::Continue),
             }),
         };
         let (ping, ping_source) = crate::sources::ping::make_ping()?;
@@ -297,16 +262,16 @@ impl<'l, Data> EventLoop<'l, Data> {
         &mut self,
         mut timeout: Option<Duration>,
         data: &mut Data,
-    ) -> io::Result<()> {
+    ) -> crate::Result<()> {
+        let now = Instant::now();
         let events = {
             let mut poll = self.handle.inner.poll.borrow_mut();
             loop {
-                let now = std::time::Instant::now();
                 let result = poll.poll(timeout);
 
                 match result {
                     Ok(events) => break events,
-                    Err(ref err) if err.kind() == io::ErrorKind::Interrupted => {
+                    Err(crate::Error::IoError(err)) if err.kind() == io::ErrorKind::Interrupted => {
                         // Interrupted by a signal. Update timeout and retry.
                         if let Some(to) = timeout {
                             let elapsed = now.elapsed();
@@ -328,16 +293,27 @@ impl<'l, Data> EventLoop<'l, Data> {
                 .inner
                 .sources
                 .borrow()
-                .get_dispatcher(event.token.id);
+                .get(event.token.key)
+                .cloned();
 
             if let Some(disp) = opt_disp {
-                let ret = disp.process_events(event.readiness, event.token, data)?;
+                let mut ret = disp.process_events(event.readiness, event.token, data)?;
+
+                // if the returned PostAction is Continue, it may be overwritten by an user-specified pending action
+                let pending_action = self
+                    .handle
+                    .inner
+                    .pending_action
+                    .replace(PostAction::Continue);
+                if let PostAction::Continue = ret {
+                    ret = pending_action;
+                }
 
                 match ret {
                     PostAction::Reregister => {
                         disp.reregister(
                             &mut self.handle.inner.poll.borrow_mut(),
-                            &mut TokenFactory::new(event.token.id),
+                            &mut TokenFactory::new(event.token.key),
                         )?;
                     }
                     PostAction::Disable => {
@@ -349,12 +325,18 @@ impl<'l, Data> EventLoop<'l, Data> {
                             .inner
                             .sources
                             .borrow_mut()
-                            .del_source(event.token.id);
+                            .remove(event.token.key);
                     }
                     PostAction::Continue => {}
                 }
 
-                if !self.handle.inner.sources.borrow().contains(event.token.id) {
+                if !self
+                    .handle
+                    .inner
+                    .sources
+                    .borrow()
+                    .contains_key(event.token.key)
+                {
                     // the source has been removed from within its callback, unregister it
                     let mut poll = self.handle.inner.poll.borrow_mut();
                     if let Err(e) = disp.unregister(&mut *poll) {
@@ -366,8 +348,8 @@ impl<'l, Data> EventLoop<'l, Data> {
                 }
             } else {
                 log::warn!(
-                    "[calloop] Received an event for non-existence source: {}",
-                    event.token.id
+                    "[calloop] Received an event for non-existence source: {:?}",
+                    event.token.key
                 );
             }
         }
@@ -382,15 +364,15 @@ impl<'l, Data> EventLoop<'l, Data> {
         }
     }
 
-    fn invoke_pre_run(&self, data: &mut Data) -> std::io::Result<()> {
-        for source in self.handle.inner.sources.borrow().iter() {
+    fn invoke_pre_run(&self, data: &mut Data) -> crate::Result<()> {
+        for (_, source) in self.handle.inner.sources.borrow().iter() {
             source.pre_run(data)?;
         }
         Ok(())
     }
 
-    fn invoke_post_run(&self, data: &mut Data) -> std::io::Result<()> {
-        for source in self.handle.inner.sources.borrow().iter() {
+    fn invoke_post_run(&self, data: &mut Data) -> crate::Result<()> {
+        for (_, source) in self.handle.inner.sources.borrow().iter() {
             source.post_run(data)?;
         }
         Ok(())
@@ -408,7 +390,7 @@ impl<'l, Data> EventLoop<'l, Data> {
         &mut self,
         timeout: D,
         data: &mut Data,
-    ) -> io::Result<()> {
+    ) -> crate::Result<()> {
         self.invoke_pre_run(data)?;
         self.dispatch_events(timeout.into(), data)?;
         self.dispatch_idles(data);
@@ -441,7 +423,7 @@ impl<'l, Data> EventLoop<'l, Data> {
         timeout: D,
         data: &mut Data,
         mut cb: F,
-    ) -> io::Result<()>
+    ) -> crate::Result<()>
     where
         F: FnMut(&mut Data),
     {
@@ -467,6 +449,7 @@ pub struct LoopSignal {
 }
 
 impl std::fmt::Debug for LoopSignal {
+    #[cfg_attr(coverage, no_coverage)]
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         f.write_str("LoopSignal { ... }")
     }
@@ -499,8 +482,8 @@ mod tests {
     use std::time::Duration;
 
     use crate::{
-        generic::Generic, ping::*, timer::Timer, Dispatcher, Interest, Mode, Poll, PostAction,
-        Readiness, RegistrationToken, Token, TokenFactory,
+        generic::Generic, ping::*, Dispatcher, Interest, Mode, Poll, PostAction, Readiness,
+        RegistrationToken, Token, TokenFactory,
     };
 
     use super::EventLoop;
@@ -516,7 +499,7 @@ mod tests {
         });
 
         event_loop
-            .dispatch(Some(Duration::from_millis(0)), &mut dispatched)
+            .dispatch(Some(Duration::ZERO), &mut dispatched)
             .unwrap();
 
         assert!(dispatched);
@@ -536,7 +519,7 @@ mod tests {
         idle.cancel();
 
         event_loop
-            .dispatch(Duration::from_millis(0), &mut dispatched)
+            .dispatch(Duration::ZERO, &mut dispatched)
             .unwrap();
 
         assert!(!dispatched);
@@ -573,38 +556,11 @@ mod tests {
         event_loop.run(None, &mut (), |_| {}).unwrap();
     }
 
-    #[test]
-    fn insert_remove() {
-        let mut event_loop = EventLoop::<()>::try_new().unwrap();
-        let dummy_token_1 = event_loop
-            .handle()
-            .insert_source(DummySource, |_, _, _| {})
-            .unwrap();
-        assert_eq!(dummy_token_1.id, 1); // numer 0 is the internal ping
-        let dummy_token_2 = event_loop
-            .handle()
-            .insert_source(DummySource, |_, _, _| {})
-            .unwrap();
-        assert_eq!(dummy_token_2.id, 2);
-        // ensure token reuse on source removal
-        event_loop.handle().remove(dummy_token_1);
-
-        event_loop
-            .dispatch(Duration::from_millis(0), &mut ())
-            .unwrap();
-
-        let dummy_token_3 = event_loop
-            .handle()
-            .insert_source(DummySource, |_, _, _| {})
-            .unwrap();
-        assert_eq!(dummy_token_3.id, 1);
-    }
-
     #[test]
     fn insert_bad_source() {
         let event_loop = EventLoop::<()>::try_new().unwrap();
         let ret = event_loop.handle().insert_source(
-            crate::sources::generic::Generic::from_fd(420, Interest::READ, Mode::Level),
+            crate::sources::generic::Generic::new(420, Interest::READ, Mode::Level),
             |_, _, _| Ok(PostAction::Continue),
         );
         assert!(ret.is_err());
@@ -614,7 +570,7 @@ mod tests {
     fn insert_source_no_interest() {
         let event_loop = EventLoop::<()>::try_new().unwrap();
         let ret = event_loop.handle().insert_source(
-            crate::sources::generic::Generic::from_fd(0, Interest::EMPTY, Mode::Level),
+            crate::sources::generic::Generic::new(0, Interest::EMPTY, Mode::Level),
             |_, _, _| Ok(PostAction::Continue),
         );
         assert!(ret.is_ok());
@@ -635,7 +591,7 @@ mod tests {
         ping.ping();
         let mut dispatched = false;
         event_loop
-            .dispatch(Duration::from_millis(0), &mut dispatched)
+            .dispatch(Duration::ZERO, &mut dispatched)
             .unwrap();
         assert!(dispatched);
 
@@ -644,7 +600,7 @@ mod tests {
         event_loop.handle().disable(&ping_token).unwrap();
         let mut dispatched = false;
         event_loop
-            .dispatch(Duration::from_millis(0), &mut dispatched)
+            .dispatch(Duration::ZERO, &mut dispatched)
             .unwrap();
         assert!(!dispatched);
 
@@ -655,7 +611,7 @@ mod tests {
         event_loop.handle().enable(&ping_token).unwrap();
         let mut dispatched = false;
         event_loop
-            .dispatch(Duration::from_millis(0), &mut dispatched)
+            .dispatch(Duration::ZERO, &mut dispatched)
             .unwrap();
         assert!(dispatched);
     }
@@ -671,13 +627,14 @@ mod tests {
             type Event = u32;
             type Metadata = ();
             type Ret = ();
+            type Error = PingError;
 
             fn process_events<F>(
                 &mut self,
                 readiness: Readiness,
                 token: Token,
                 mut callback: F,
-            ) -> std::io::Result<PostAction>
+            ) -> Result<PostAction, Self::Error>
             where
                 F: FnMut(Self::Event, &mut Self::Metadata) -> Self::Ret,
             {
@@ -692,7 +649,7 @@ mod tests {
                 &mut self,
                 poll: &mut Poll,
                 token_factory: &mut TokenFactory,
-            ) -> std::io::Result<()> {
+            ) -> crate::Result<()> {
                 self.ping1.register(poll, token_factory)?;
                 self.ping2.register(poll, token_factory)?;
                 Ok(())
@@ -702,13 +659,13 @@ mod tests {
                 &mut self,
                 poll: &mut Poll,
                 token_factory: &mut TokenFactory,
-            ) -> std::io::Result<()> {
+            ) -> crate::Result<()> {
                 self.ping1.reregister(poll, token_factory)?;
                 self.ping2.reregister(poll, token_factory)?;
                 Ok(())
             }
 
-            fn unregister(&mut self, poll: &mut Poll) -> std::io::Result<()> {
+            fn unregister(&mut self, poll: &mut Poll) -> crate::Result<()> {
                 self.ping1.unregister(poll)?;
                 self.ping2.unregister(poll)?;
                 Ok(())
@@ -736,14 +693,14 @@ mod tests {
         let mut dispatched = 0;
         ping1.ping();
         event_loop
-            .dispatch(Duration::from_millis(0), &mut dispatched)
+            .dispatch(Duration::ZERO, &mut dispatched)
             .unwrap();
         assert_eq!(dispatched, 1);
 
         dispatched = 0;
         ping2.ping();
         event_loop
-            .dispatch(Duration::from_millis(0), &mut dispatched)
+            .dispatch(Duration::ZERO, &mut dispatched)
             .unwrap();
         assert_eq!(dispatched, 2);
 
@@ -751,32 +708,32 @@ mod tests {
         ping1.ping();
         ping2.ping();
         event_loop
-            .dispatch(Duration::from_millis(0), &mut dispatched)
+            .dispatch(Duration::ZERO, &mut dispatched)
             .unwrap();
         assert_eq!(dispatched, 3);
     }
 
     #[test]
     fn change_interests() {
-        use nix::sys::socket::{socketpair, AddressFamily, SockFlag, SockType};
-        use nix::unistd::{read, write};
+        use nix::sys::socket::{recv, socketpair, AddressFamily, MsgFlags, SockFlag, SockType};
+        use nix::unistd::write;
         let mut event_loop = EventLoop::<bool>::try_new().unwrap();
 
         let (sock1, sock2) = socketpair(
             AddressFamily::Unix,
             SockType::Stream,
             None,
-            SockFlag::SOCK_NONBLOCK,
+            SockFlag::empty(), // recv with DONTWAIT will suffice for platforms without SockFlag::SOCK_NONBLOCKING such as macOS
         )
         .unwrap();
 
-        let source = Generic::from_fd(sock1, Interest::READ, Mode::Level);
-        let dispatcher = Dispatcher::new(source, |_, fd, dispatched| {
+        let source = Generic::new(sock1, Interest::READ, Mode::Level);
+        let dispatcher = Dispatcher::new(source, |_, &mut fd, dispatched| {
             *dispatched = true;
             // read all contents available to drain the socket
             let mut buf = [0u8; 32];
             loop {
-                match read(fd.0, &mut buf) {
+                match recv(fd, &mut buf, MsgFlags::MSG_DONTWAIT) {
                     Ok(0) => break, // closed pipe, we are now inert
                     Ok(_) => {}
                     Err(e) => {
@@ -802,7 +759,7 @@ mod tests {
         // first dispatch, nothing is readable
         let mut dispatched = false;
         event_loop
-            .dispatch(Duration::from_millis(0), &mut dispatched)
+            .dispatch(Duration::ZERO, &mut dispatched)
             .unwrap();
         assert!(!dispatched);
 
@@ -810,14 +767,14 @@ mod tests {
         write(sock2, &[1, 2, 3]).unwrap();
         dispatched = false;
         event_loop
-            .dispatch(Duration::from_millis(0), &mut dispatched)
+            .dispatch(Duration::ZERO, &mut dispatched)
             .unwrap();
         assert!(dispatched);
 
         // All has been read, no longer readable
         dispatched = false;
         event_loop
-            .dispatch(Duration::from_millis(0), &mut dispatched)
+            .dispatch(Duration::ZERO, &mut dispatched)
             .unwrap();
         assert!(!dispatched);
 
@@ -828,7 +785,7 @@ mod tests {
         // the socket is writable
         dispatched = false;
         event_loop
-            .dispatch(Duration::from_millis(0), &mut dispatched)
+            .dispatch(Duration::ZERO, &mut dispatched)
             .unwrap();
         assert!(dispatched);
 
@@ -839,7 +796,7 @@ mod tests {
         // the socket is not readable
         dispatched = false;
         event_loop
-            .dispatch(Duration::from_millis(0), &mut dispatched)
+            .dispatch(Duration::ZERO, &mut dispatched)
             .unwrap();
         assert!(!dispatched);
     }
@@ -854,7 +811,7 @@ mod tests {
             .handle()
             .insert_source(ping_source, move |(), &mut (), opt_src| {
                 if let Some(src) = opt_src.take() {
-                    handle.kill(src);
+                    handle.remove(src);
                 }
             })
             .unwrap();
@@ -863,80 +820,11 @@ mod tests {
 
         let mut opt_src = Some(ping_token);
 
-        event_loop
-            .dispatch(Duration::from_millis(0), &mut opt_src)
-            .unwrap();
+        event_loop.dispatch(Duration::ZERO, &mut opt_src).unwrap();
 
         assert!(opt_src.is_none());
     }
 
-    #[test]
-    fn non_static_source() {
-        let mut flag = false;
-
-        {
-            let timer = Timer::new().unwrap();
-            let handle = timer.handle();
-            handle.add_timeout(std::time::Duration::from_millis(5), &mut flag);
-
-            let mut fired = false;
-
-            let mut event_loop = EventLoop::<bool>::try_new().unwrap();
-            let _timer_token = event_loop
-                .handle()
-                .insert_source(timer, |flag, _, fired| {
-                    *flag = true;
-                    *fired = true;
-                })
-                .unwrap();
-
-            event_loop
-                .dispatch(Duration::from_millis(500), &mut fired)
-                .unwrap();
-
-            assert!(fired);
-        }
-
-        assert!(flag);
-    }
-
-    #[test]
-    fn non_static_dispatcher() {
-        let mut flag = false;
-
-        {
-            let timer = Timer::<&mut bool>::new().unwrap();
-            let _ = timer
-                .handle()
-                .add_timeout(std::time::Duration::from_millis(5), &mut flag);
-
-            let dispatcher = Dispatcher::new(timer, |flag, _, fired| {
-                *flag = true;
-                *fired = true;
-            });
-
-            let mut fired = false;
-
-            let mut event_loop = EventLoop::<bool>::try_new().unwrap();
-            let disp_token = event_loop
-                .handle()
-                .register_dispatcher(dispatcher.clone())
-                .unwrap();
-
-            event_loop
-                .dispatch(Duration::from_millis(500), &mut fired)
-                .unwrap();
-            assert!(fired);
-
-            event_loop.handle().remove(disp_token);
-
-            let timer = dispatcher.into_source_inner();
-            timer.handle().cancel_all_timeouts();
-        }
-
-        assert!(flag);
-    }
-
     #[test]
     fn non_static_data() {
         use std::sync::mpsc;
@@ -959,7 +847,7 @@ mod tests {
             ping.ping();
 
             event_loop
-                .dispatch(Duration::from_millis(0), &mut ref_sender)
+                .dispatch(Duration::ZERO, &mut ref_sender)
                 .unwrap();
         }
 
@@ -975,13 +863,14 @@ mod tests {
         type Event = ();
         type Metadata = ();
         type Ret = ();
+        type Error = crate::Error;
 
         fn process_events<F>(
             &mut self,
             _: Readiness,
             _: Token,
             mut callback: F,
-        ) -> std::io::Result<PostAction>
+        ) -> Result<PostAction, Self::Error>
         where
             F: FnMut(Self::Event, &mut Self::Metadata) -> Self::Ret,
         {
@@ -989,15 +878,15 @@ mod tests {
             Ok(PostAction::Continue)
         }
 
-        fn register(&mut self, _: &mut Poll, _: &mut TokenFactory) -> std::io::Result<()> {
+        fn register(&mut self, _: &mut Poll, _: &mut TokenFactory) -> crate::Result<()> {
             Ok(())
         }
 
-        fn reregister(&mut self, _: &mut Poll, _: &mut TokenFactory) -> std::io::Result<()> {
+        fn reregister(&mut self, _: &mut Poll, _: &mut TokenFactory) -> crate::Result<()> {
             Ok(())
         }
 
-        fn unregister(&mut self, _: &mut Poll) -> std::io::Result<()> {
+        fn unregister(&mut self, _: &mut Poll) -> crate::Result<()> {
             Ok(())
         }
     }
